from libcpp cimport bool
from libcpp.string cimport string


cdef extern from "mixnet.h":
    bool key_gen(const long votes_number, string  public_file, string  secret_file)


cdef extern from "mixnet.h":
    bool create_crs(const long votes_number, string  crs_file, string  public_file)


cdef extern from "mixnet.h":
    bool generate_encoded_votes(string  crs_file, string  votes_file)


cdef extern from "mixnet.h":
    bool demo_voting(const long votes_number, const long encoding_number,
                     string public_file, string votes_file)


cdef extern from "mixnet.h":
    bool encrypt(string  crs_file, string  votes_file, string  ciphertexts_file)


cdef extern from "mixnet.h":
    bool prove(string  crs_file, string  ciphertexts_file, string  proofs_file)


cdef extern from "mixnet.h":
    bool verify(string  crs_file, string  ciphertexts_file, string  proofs_file)


cdef extern from "mixnet.h":
    bool decrypt(string  crs_file, string  votes_file, string  proofs_file,
                 string  decrypted_votes_file, string  secret_file)


cdef extern from "mixnet.h":
    bool test_mixnet(const long n)



cimport hatshufflepy
from cpython.version cimport PY_MAJOR_VERSION


cdef unicode _ustring(s):
    if type(s) is unicode:
        # fast path for most common case(s)
        return <unicode>s
    elif PY_MAJOR_VERSION < 3 and isinstance(s, bytes):
        # only accept byte strings in Python 2.x, not in Py3
        return (<bytes>s).decode('ascii')
    elif isinstance(s, unicode):
        # an evil cast to <unicode> might work here in some(!) cases,
        # depending on what the further processing does.  to be safe,
        # we can always create a copy instead
        return unicode(s)
    else:
        raise TypeError(...)


def hat_shuffle_key_gen(votes_number, public_file, secret_file):
    """Generate El Gamal key pair

    Args:
        votes_number: Number of votes to mix
        public_file: JSON file to save public key
        secret_file: File to save secret key
    """
    public_file_ = public_file.encode('utf-8')
    secret_file_ = secret_file.encode('utf-8')
    hatshufflepy.key_gen(votes_number, public_file_, secret_file_)


def hat_shuffle_create_crs(votes_number, crs_file, public_file):
    """Generate Common Reference string

    Args:
        votes_number: Number of votes to mix
        crs_file: JSON file to save crs
        public_file: Public Key file
    """
    crs_file_ = crs_file.encode('utf-8')
    public_file_ = public_file.encode('utf-8')
    hatshufflepy.create_crs(votes_number, crs_file_, public_file_)



def hat_shuffle_demo_voting(votes_number, encoding_number, public_file,
                            votes_file):
    """Generate encoded votes in a specific range

    Args:
        votes_number: Number of votes to generate
        encoding_number: Number of different encodings
        public_file: JSON file to get public key
        votes_file: JSON file to save votes
    """
    public_file_ = public_file.encode('utf-8')
    votes_file_ = votes_file.encode('utf-8')
    hatshufflepy.demo_voting(votes_number, encoding_number, public_file_,
                             votes_file_)


def hat_shuffle_generate_encoded_votes(crs_file, votes_file):
    """Generate encoded votes for testing purposes

    Args:
        crs_file: JSON crs file
        votes_file: JSON file to save votes
    """
    crs_file_ = crs_file.encode('utf-8')
    votes_file_ = votes_file.encode('utf-8')
    hatshufflepy.generate_encoded_votes(crs_file_, votes_file_)


def hat_shuffle_encrypt(crs_file, votes_file, ciphertexts_file):
    """Encrypt Votes

    Args:
        crs_file: JSON crs file
        votes_file: JSON votes file
        ciphertexts_file: JSON file to save ciphertexts
    """
    crs_file_ = crs_file.encode('utf-8')
    votes_file_ = votes_file.encode('utf-8')
    ciphertexts_file_ = ciphertexts_file.encode('utf-8')
    hatshufflepy.encrypt(crs_file_, votes_file_, ciphertexts_file_)


def hat_shuffle_prove(crs_file, ciphertexts_file, proofs_file):
    """Proving

    Args:
        crs_file: JSON crs file
        ciphertexts_file: JSON ciphertexts file
        proofs_file: JSON file to save proofs
    """
    crs_file_ = crs_file.encode('utf-8')
    proofs_file_ = proofs_file.encode('utf-8')
    ciphertexts_file_ = ciphertexts_file.encode('utf-8')
    hatshufflepy.prove(crs_file_, ciphertexts_file_, proofs_file_)


def hat_shuffle_verify(crs_file, ciphertexts_file, proofs_file):
    """Verifying

    Args:
        crs_file: JSON crs file
        ciphertexts_file: JSON ciphertexts file
        proofs_file: JSON proofs file
    Returns:
        True if verification was successful, otherwise returns False
    """
    crs_file_ = crs_file.encode('utf-8')
    proofs_file_ = proofs_file.encode('utf-8')
    ciphertexts_file_ = ciphertexts_file.encode('utf-8')
    return hatshufflepy.verify(crs_file_, ciphertexts_file_, proofs_file_)


def hat_shuffle_decrypt(crs_file, votes_file, proofs_file,
                        decrypted_votes_file, secret_file):
    """Decryption of shuffled ciphertexts

    Args:
        crs_file: JSON crs file
        votes_file: JSON votes file
        proofs_file: JSON proofs file
        secret_file: Secret key filee
        decrypted_votes_file: JSON file to save decrypted votes
    """
    crs_file_ = crs_file.encode('utf-8')
    proofs_file_ = proofs_file.encode('utf-8')
    votes_file_ = votes_file.encode('utf-8')
    secret_file_ = secret_file.encode('utf-8')
    decrypted_votes_file_ = decrypted_votes_file.encode('utf-8')
    hatshufflepy.decrypt(crs_file_, votes_file_, proofs_file_,
                         decrypted_votes_file_, secret_file_)


def hat_shuffle_test_mixnet(n):
    hatshufflepy.test_mixnet(n)



import os
import sys
import shutil
from setuptools import setup, find_packages
from distutils.sysconfig import get_python_lib
from subprocess import call
from setuptools.command.install import install
from distutils.extension import Extension
from Cython.Build import cythonize
from Cython.Distutils import build_ext


#  os.environ['CC'] = 'g++'
#  os.environ['CXX'] = 'g++'

major = str(sys.version_info.major)
minor = str(sys.version_info.minor)

ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
VERSION = 'python' + major + '.' + minor
LIB_PATH = get_python_lib()
print(ROOT_PATH)


class CustomBuild(install):
    def run(self):
        call(['bash', 'install_depends.sh'])
        install.run(self)


class MyBuildExt(build_ext):
    def run(self):
        build_ext.run(self)
        build_dir = os.path.realpath(self.build_lib)
        root_dir = os.path.dirname(os.path.realpath(__file__))
        target_dir = build_dir if not self.inplace else root_dir
        if not os.path.exists(target_dir + '/hatshufflepy'):
            os.makedirs(target_dir + '/hatshufflepy')
        self.copy_file('hatshufflepy/libhatshuffle/lib/libhatshuffle.so',
                       root_dir, target_dir + '/hatshufflepy/libhatshuffle.so')
        self.copy_file('hatshufflepy/libhatshuffle/lib/libff.so',
                       root_dir, target_dir + '/hatshufflepy/libff.so')
        self.copy_file('hatshufflepy/libhatshuffle/lib/libzm.so',
                       root_dir, target_dir + '/hatshufflepy/libzm.so')

    def copy_file(self, path, source_dir, destination_dir):
        if os.path.exists(os.path.join(source_dir, path)):
            shutil.copyfile(os.path.join(source_dir, path),
                            destination_dir)


module = Extension('hatshufflepy',
                   sources=['hatshufflepy/hatshufflepy.pyx'],
                   language='c++',
                   include_dirs=['hatshufflepy/libhatshuffle/include/libhatshuffle',
                                 'hatshufflepy/libhatshuffle/include',
                                 'libhatshuffle/include/libff'],
                   libraries=['zm', 'ff', 'hatshuffle'],
                   library_dirs=['/usr/local/lib',
                                 'hatshufflepy/libhatshuffle/lib'],
                   runtime_library_dirs=[ROOT_PATH +
                                         '/hatshufflepy/libhatshuffle/lib',
					 '/usr/local/lib/' + VERSION + #  ubuntu
					 '/dist-packages/hatshufflepy',
					 LIB_PATH + '/hatshufflepy'],
                   extra_compile_args=['-std=c++11', '-fPIC',
                                       '-shared', '-w',
                                       '-static', '-O3'],
                   extra_link_args=['-lgmp', '-fopenmp', '-g'])


setup(
    name='hatshufflepy',
    version='0.0.6',
    packages=find_packages(),
    author='Stefanos Chaliasos',
    author_email='stefanoshaliassos@gmail.com',
    description='Cython wrapper for hat shuffle implementation',
    license='MIT',
    keywords='mixnet ecc',
    url='https://github.com/StefanosChaliasos/hatshufflepy',

    install_requires=["cython >= 0.22.1"],
    ext_modules=cythonize(module),
    cmdclass={'build_ext': MyBuildExt, 'install': CustomBuild},
)
